package cn.com.infosec.mobile.android.framework.crypto.impl.gm;

import cn.com.infosec.asn1.x9.X9IntegerConverter;
import cn.com.infosec.math.ec.ECConstants;
import cn.com.infosec.math.ec.ECCurve;
import cn.com.infosec.math.ec.ECFieldElement;
import cn.com.infosec.math.ec.ECPoint;
import java.math.BigInteger;

public abstract class JPECPoint {
   private static X9IntegerConverter converter = new X9IntegerConverter();
   ECCurve curve;
   ECFieldElement x;
   ECFieldElement y;
   ECFieldElement z;
   protected boolean withCompression;
   protected JPECPoint infinity = null;
   protected JPECMultiplier multiplier = null;
   protected JPPreCompInfo preCompInfo = null;

   protected JPECPoint(ECCurve curve, ECFieldElement x, ECFieldElement y, ECFieldElement z) {
      this.curve = curve;
      this.x = x;
      this.y = y;
      this.z = z;
   }

   public JPECPoint(ECPoint point) {
      if (point.isInfinity()) {
         this.curve = null;
         this.x = null;
         this.y = null;
         this.z = null;
      } else {
         this.curve = point.getCurve();
         this.x = point.getX();
         this.y = point.getY();
         this.z = new ECFieldElement.Fp(SM2GMParameter.gmp, BigInteger.ONE);
      }

   }

   public ECCurve getCurve() {
      return this.curve;
   }

   public ECFieldElement getX() {
      return this.x;
   }

   public ECFieldElement getY() {
      return this.y;
   }

   public ECFieldElement getZ() {
      return this.z;
   }

   public boolean isInfinity() {
      return this.x == null && this.y == null && this.z == null;
   }

   public JPECPoint getInfinity() {
      return new Fp(SM2GMParameter.gmec256.getInfinity());
   }

   public boolean isCompressed() {
      return this.withCompression;
   }

   public boolean equals(Object other) {
      if (other == this) {
         return true;
      } else if (!(other instanceof JPECPoint)) {
         return false;
      } else {
         JPECPoint other1 = (JPECPoint)other;
         if (this.isInfinity()) {
            return other1.isInfinity();
         } else {
            return this.x.equals(other1.x) && this.y.equals(other1.y);
         }
      }
   }

   public int hashCode() {
      return this.isInfinity() ? 0 : this.x.hashCode() ^ this.y.hashCode();
   }

   void setPreCompInfo(JPPreCompInfo preCompInfo) {
      this.preCompInfo = preCompInfo;
   }

   public abstract byte[] getEncoded();

   public abstract JPECPoint add(JPECPoint var1);

   public abstract JPECPoint subtract(JPECPoint var1);

   public abstract JPECPoint negate();

   public abstract JPECPoint twice();

   synchronized void assertECMultiplier() {
      if (this.multiplier == null) {
         JPECPoint var10000 = this;
         JPWNafMultiplier var1;
         var1 = new JPWNafMultiplier.<init>();
         var10000.multiplier = var1;
      }

   }

   ECPoint toECPoint() {
      JPECPoint var10000 = this;
      JPECPoint var10001 = this;
      BigInteger var1;
      BigInteger var4;
      BigInteger var10002 = var1 = this.z.toBigInteger().modInverse(var4 = SM2GMParameter.gmp);
      BigInteger var2 = var10002.multiply(var10002);
      BigInteger var3 = var10001.x.toBigInteger().multiply(var2).mod(var4);
      var4 = var10000.y.toBigInteger().multiply(var1).multiply(var2).mod(var4);
      return SM2GMParameter.gmec256.createPoint(var3, var4, false);
   }

   public JPECPoint multiply(BigInteger k) {
      if (k.signum() >= 0) {
         if (this.isInfinity()) {
            return this;
         } else if (k.signum() == 0) {
            return this.infinity;
         } else {
            this.assertECMultiplier();
            return this.multiplier.multiply(this, k, this.preCompInfo);
         }
      } else {
         throw new IllegalArgumentException("The multiplicator cannot be negative");
      }
   }

   public static class Fp extends JPECPoint {
      public Fp(ECCurve curve, ECFieldElement x, ECFieldElement y, ECFieldElement z) {
         this(curve, x, y, z, false);
      }

      public Fp(ECCurve curve, ECFieldElement x, ECFieldElement y) {
         Fp var10000 = this;
         ECFieldElement var4 = SM2Constants.ONE;
         var10000.<init>(curve, x, y, var4, false);
      }

      public Fp(ECPoint point) {
         super(point);
      }

      public Fp(ECCurve curve, ECFieldElement x, ECFieldElement y, ECFieldElement z, boolean withCompression) {
         super(curve, x, y, z);
         if ((x == null || y != null) && (x != null || y == null)) {
            super.withCompression = withCompression;
         } else {
            throw new IllegalArgumentException("Exactly one of the field elements is null");
         }
      }

      public byte[] getEncoded() {
         if (this.isInfinity()) {
            return new byte[1];
         } else {
            Fp var10000 = this;
            ECPoint var3 = this.toECPoint();
            int var1 = JPECPoint.converter.getByteLength(var3.getX());
            byte[] var4;
            int var5;
            byte[] var6;
            byte[] var9;
            byte[] var10001;
            if (var10000.withCompression) {
               byte var8;
               if (var3.getY().toBigInteger().testBit(0)) {
                  var8 = 3;
               } else {
                  var8 = 2;
               }

               var9 = var6 = new byte[(var4 = JPECPoint.converter.integerToBytes(var3.getX().toBigInteger(), var1)).length + 1];
               var10001 = var4;
               var6[0] = var8;
               var5 = var4.length;
               System.arraycopy(var10001, 0, var6, 1, var5);
               return var9;
            } else {
               byte[] var2;
               var9 = var2 = JPECPoint.converter.integerToBytes(var3.getX().toBigInteger(), var1);
               var4 = JPECPoint.converter.integerToBytes(var3.getY().toBigInteger(), var1);
               var9 = var6 = new byte[var9.length + var4.length + 1];
               var10001 = var4;
               byte[] var10002 = var4;
               var6[0] = 4;
               var5 = var2.length;
               System.arraycopy(var2, 0, var6, 1, var5);
               var5 = var2.length + 1;
               int var7 = var10002.length;
               System.arraycopy(var10001, 0, var6, var5, var7);
               return var9;
            }
         }
      }

      public JPECPoint add(JPECPoint b) {
         if (this.isInfinity()) {
            return b;
         } else if (b.isInfinity()) {
            return this;
         } else if (super.x.equals(b.x)) {
            return super.y.equals(b.y) ? this.twice() : this.getInfinity();
         } else {
            ECFieldElement var2;
            ECFieldElement var10000 = (var2 = super.z.square()).multiply(super.z);
            var2 = var2.multiply(b.getX());
            var10000 = var10000.multiply(b.getY());
            var2 = var2.subtract(super.x);
            ECFieldElement var3 = var10000.subtract(this.getY());
            BigInteger var4;
            if ((var4 = ECConstants.ZERO).equals(var2.toBigInteger())) {
               return var4.equals(var3.toBigInteger()) ? b.twice() : this.getInfinity();
            } else {
               var2 = super.z.square();
               var3 = b.getZ().square();
               ECFieldElement var10001 = super.x.multiply(var3);
               ECFieldElement var8;
               ECFieldElement var5 = var10001.subtract(var8 = b.getX().multiply(var2));
               ECFieldElement var10002 = var8;
               var8 = (var3 = super.y.multiply(var3).multiply(b.getZ())).subtract(var2 = b.getY().multiply(var2).multiply(super.z));
               ECFieldElement var6;
               var10001 = var6 = var10001.add(var10002);
               var2 = var3.add(var2);
               var3 = var8.square().subtract(var6.multiply(var5.square()));
               var2 = var10001.multiply(var5.square()).subtract(var3).subtract(var3).multiply(var8).subtract(var2.multiply(var5.square()).multiply(var5)).divide(SM2Constants.TWO);
               ECFieldElement b1;
               return (JPECPoint)((b1 = super.z.multiply(b.getZ()).multiply(var5)).toBigInteger().compareTo(BigInteger.ZERO) == 0 ? this.getInfinity() : new Fp(super.curve, var3, var2, b1));
            }
         }
      }

      public JPECPoint twice() {
         if (this.isInfinity()) {
            return this;
         } else {
            ECFieldElement var1;
            ECFieldElement var10000 = var1 = SM2Constants.THREE.multiply(super.x.square()).add(super.z.square().square().multiply(SM2Constants.A));
            ECFieldElement var2 = SM2Constants.FOUR.multiply(super.x).multiply(super.y.square());
            ECFieldElement var3 = SM2Constants.EIGHT.multiply(super.y.square().square());
            ECFieldElement var4 = var10000.square();
            ECFieldElement var5;
            var10000 = var5 = SM2Constants.TWO;
            var1 = var2.subtract(var2 = var4.subtract(var5.multiply(var2))).multiply(var1).subtract(var3);
            var3 = var10000.multiply(super.y).multiply(super.z);
            return new Fp(super.curve, var2, var1, var3);
         }
      }

      public JPECPoint subtract(JPECPoint b) {
         return (JPECPoint)(b.isInfinity() ? this : this.add(b.negate()));
      }

      public JPECPoint negate() {
         Fp var10002 = this;
         Fp var10003 = this;
         Fp var10004 = this;
         ECCurve var4 = super.curve;
         ECFieldElement var1 = var10004.x;
         ECFieldElement var2 = var10003.y.negate();
         ECFieldElement var3 = var10002.z;
         return new Fp(var4, var1, var2, var3);
      }

      synchronized void assertECMultiplier() {
         if (super.multiplier == null) {
            Fp var10000 = this;
            JPWNafMultiplier var1;
            var1 = new JPWNafMultiplier.<init>();
            var10000.multiplier = var1;
         }

      }
   }
}
